##############################################################################
#
# Copyright (C) Zenoss, Inc. 2013, all rights reserved.
#
# This content is made available according to terms specified in
# License.zenoss under the directory where your Zenoss product is installed.
#
##############################################################################

from zope.component import adapts
from zope.interface import implements

from Products.ZenRelations.RelSchema import ToMany, ToManyCont, ToOne

from Products.Zuul.catalog.paths import DefaultPathReporter
from Products.Zuul.decorators import info
from Products.Zuul.form import schema
from Products.Zuul.infos import ProxyProperty
from Products.Zuul.infos.component import ComponentInfo
from Products.Zuul.interfaces.component import IComponentInfo
from Products.Zuul.utils import ZuulMessageFactory as _t

from ZenPacks.zenoss.AWS import MODULE_NAME
from ZenPacks.zenoss.AWS.AWSComponent import AWSComponent


class S3Bucket(AWSComponent):
    '''
    Model class for S3Bucket.
    '''

    meta_type = portal_type = 'S3Bucket'

    creation_date = None

    # Note: At this time, region is stored as a property rather than a
    # relationship to EC2Region because it is possible to have credentials
    # that do not permit the aws.EC2 modeler to complete, which leaves
    # the regions unmodeled.   This may be worth revisiting in the future.
    region = None

    _properties = AWSComponent._properties + (
        {'id': 'creation_date', 'type': 'string'},
        {'id': 'region', 'type': 'string'},
        )

    _relations = AWSComponent._relations + (
        ('account', ToOne(
            ToManyCont, MODULE_NAME['EC2Account'], 's3buckets')),
        )

    def getRegionId(self):
        if self.region:
            return self.region
        else:
            return ""


class IS3BucketInfo(IComponentInfo):
    '''
    API Info interface for S3Bucket.
    '''

    creation_date = schema.TextLine(title=_t(u'Creation date'))
    region = schema.TextLine(title=_t(u'Region'))
    account = schema.Entity(title=_t(u'Account'))


class S3BucketInfo(ComponentInfo):
    '''
    API Info adapter factory for S3Bucket.
    '''

    implements(IS3BucketInfo)
    adapts(S3Bucket)

    creation_date = ProxyProperty('creation_date')
    region = ProxyProperty('region')

    @property
    @info
    def account(self):
        return self._object.device()
